// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.


/**
 * @file adc.c
 */

#include <xc.h>
#include <stdint.h>
#include "adc.h"
#include "common.h"

/**
 * Available ADC channels to read from
 *
 * SPEED = Motor Speed ( 0, 20 -> 120Hz ) in both directions
 * CURRENT_PINCH = Adjust point to shut-off the motor case of over current
 * PUSH_BUTTONS = Two push buttons for motor toggle
 * CURRENT_SENSE = Motor feedback
 * AUX_INPUTS = Auxillary isolated inputs
 */
typedef enum {
    SPEED,
    CURRENT_PINCH,
    PUSH_BUTTONS,
    CURRENT_SENSE,
    AUX_INPUTS,

} ADC_INPUTS;

#define SPEED_CHANNEL    0b0110
#define OVER_CURRENT_CHANNEL    0b0000
#define BUTTONS_CHANNEL     0b0011
#define SENSE_CHANNEL    0b1000
#define AUX_CHANNEL     0b1011


static uint16_t ReadADC(uint8_t channel);

uint16_t GetSpeed() {
    return ReadADC(SPEED_CHANNEL);
}

uint16_t GetCurrentPinch() {
    return ReadADC(OVER_CURRENT_CHANNEL);
}

uint16_t GetButtons() {
    return ReadADC(BUTTONS_CHANNEL);
}

uint16_t GetCurrentSense() {
    return ReadADC(SENSE_CHANNEL);
}

uint16_t GetAuxInputs() {
    return ReadADC(AUX_CHANNEL);
}

void SetupADC() {
    ADCON1 = 0xA0; //right justified, Fosc/32 speed
    ADCON0 = 0b00011001; //RC2/AN6 is source , ADC ON. ADC SRC pin is changed frequently
}

/**
 * Read ADC value from the following:
 *
 * Isolated speed/current potentiatmotrs
 * Isolated buttons
 * Isolated auxillary inputs
 * current sense
 *
 * @note All of the isolated ADC readings will have a Voltage drop due to the
 *       opto-coupler's diode voltage drop (approx 300mV). Must compensate for this.
 *
 * @todo Compensate for Voltage drops on opto-couplers
 *
 * @return int 10-bit ADC value
 */
static uint16_t ReadADC(uint8_t channel) {
    ADCON0bits.CHS = channel; // AN6
   // __delay_us(10); //Wait for when changing channels
    ADGO = 1; // Start the ADC conversion
    while (ADGO); //wait for conversion to end
    return ((ADRESH << 8) + ADRESL); // Return the full 10-bit ADC reading
    //  return 1023;
}