// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.
/**
 * @file common.h
 *
 * Place any system settings in this file
 *
 * These are the only #defines that should be changed without consequences
 *
 */

#ifndef __COMMON_H_
#define __COMMON_H_

// The only software difference between revisions is the LED configuration
#define REV_NUM         2 //Rev options are: 2(b) and 3(c). Note that v1.0 on silkscreen
                          // designates Revb.

/**
 * Commenting this #define will disable CLC4 from outputing PWM4 on P2. 
 * Please see the acceptance testing document in creating a simple circuit 
 * to verify the PWM output
 */
#define DEBUG

// TMR1 is used as generic timer and multiple items rely on this as a heartbeat
#define TMR1_TIMER_SET           10 // (ms) - MAX is 16ms without needing to change prescaler

/**
 * How often Speed and Current Pinch POTs are sampled as a function of TMR1_TIMER_SET (ms)
 */
#define SAMPLE_POT_RATE 10 // UINT16_MAX


/**
 * Other frequencies can be used, however 15.6KHz is near the limit because
 * the CPU must perform expensive multiplication and addition in order to
 * calculate the resultant sine wave. These frequencies were chosen since
 * it did not require any change to the oringal sine lookup table and the math
 * division/multiplication are powers of 2.
 *
 * @note A frequency of 15.6KHz places a large strain on the CPU to constantly
 * be in the TMR2 ISR loading new PWM values. Because of this, the board will
 * become less responsive as the freq is increased. SoftStart/Stop routine will
 * also increase in time execution due to the CPU being in the ISR all the time
 * and not performing the calculations.
 *
 *
 */
#define FREQ_3906   3906    // 3.9KHz
#define FREQ_7751   7812    // 7.8KHz
#define FREQ_15625   15625  // 15.625KHz

#define SET_FREQ   FREQ_7751

/**
 * The board is capable of driving a 3-phase motor or 1-phase permanent-split
 * capacitor run motor (PSC). User must specify the type in 'MOTOR_TYPE'
 *
 */
#define SINGLE_PHASE 0
#define THREE_PHASE  1

#define MOTOR_TYPE THREE_PHASE // SINGLE_PHASE or THREE_PHASE allowed here



/**
 * Amount of times Timer1 rolls over before incrementing Soft Start V/F profile
 *
 * Soft Start works by slowly increasing the sine wave frequency and magnitude
 * This routine can be extended or shortened
 *
 * @warning The limitiation to the SoftStart/Stop delay is calculating
 * all of the values in the sineTable in LoadModulateBuffer()
 *
 */
#define SOFT_START_DURATION  50 // Amount of SoftStart calls until finished - UINT8_MAX
#define SOFT_START_DELAY    2 // TMR1_TIMER_SET (ms) per each count - UINT16_MAX

#define SOFT_STOP_DURATION  50 // UINT8_MAX
#define SOFT_STOP_DELAY    2 // UINT8_MAX

/**
 * Motor can optionally be programed to brake upon a button press
 * This will force the complement input PWM pins to remain HIGH whilst the PWM
 * pins held LOW. This will cause the motor to violently come to a halt. Leave
 * this line commented out to have the motor wind down instead.
 */
//#define MOTOR_BRAKE

/**
 * All in units of Hz
 *
 * This is what the Soft Start algorithm uses. It starts at the Min and ends at Max.
 * The Amplitude of the sine wave hits its max at Ideal. This is from the V/F algorithm
 */
#define MODULATION_MAX      120 // Max frequency (@warning - Changing this from 120 or 60Hz will require
                                // Floating point math calculations that have not been implemented in
                                // in 'CalcPhase' in demoboard.c
#define MODULATION_IDEAL    60  // ideal motor speed
#define MODULATION_MIN      25  // min


/**
 * Current Trip points
 *
 * Main line code is constantly reading the current sense
 */
#define CURRENT_TRIP_V       2 // Volts where to trip upon default
#define CURRENT_TRIP_AVG     32 // Number of ADC measurements to Average - Make this a multiple of 2 for code speed


// Init start LEDs
#define START_BLINKS 5 // Number of times LED blink in POR
#define START_DELAY  100 // Time in-between START_BLINKs (in ms)



/**************************************
 * ADC
 **************************************/

/**
 * ADC values used to determine what I/O is active.
 *
 * The auxillary inputs and push buttons are multiplexed. The following
 * calculates the minimum voltage levels that the resultant ADV value can
 * be compared against in order to obtain what input was used after IOC
 *
 * Input values can be decimal
 */
#define BOARD_V             5 // PIC Vdd voltage
#define ADC_REF             BOARD_V  // Vdd as ref

/**
 * Analog voltages for external switches
 */
#define S1_V                4.8 // Expected: 4.8V
#define S2_V                3.3 // Expect:   3.3V

// AUX1 is a dedicated digital Input. It is not multiplexed!
// #define AUX1_V

#define AUX2_V              2.0
#define AUX3_V              3.5
#define AUX4_V              4.0

/**
 * @warning Changing the frequency will alter all timers, including the PWM
 */
#define _XTAL_FREQ 16000000 // 16MHz speed - must also change this for OSCCON
#if (_XTAL_FREQ == 16000000)
#define IRCF_VAL  0b1111
#else
#error wrong frequency selected
#endif


/**
 * DO NOT EDIT BELOW THIS COMMENT BLOCK.
 * The following protect from incorrect inputs
 */

#if(TMR1_TIMER_SET > 16)
#error  0 < TMR1_TIMER_SET < 16
#endif
#if(SAMPLE_POT_RATE > UINT8_MAX)
#error 0 < SAMPLE_POT_RATE < 2^8
#endif
#if(SOFT_START_DURATION > UINT8_MAX)
#error 0 < SOFT_START_DURATION < 2^8
#endif
#if(SOFT_START_DELAY > UINT8_MAX)
#error 0 < SOFT_START_DELAY < 2^8
#endif

#if((MODULATION_MAX > 120) || MODULATION_MAX < MODULATION_MIN)
#error MODULATION_MIN < MODULATION_MAX < 120
#endif
#if((MODULATION_IDEAL > MODULATION_MAX) || MODULATION_IDEAL < MODULATION_MIN)
#error MODULATION_MIN < MODULATION_IDEAL < MODULATION_MAX
#endif

#if((BOARD_V > 5) || (BOARD_V < 3))
#error 3 < BOARD_V < 5
#endif

#if ((SET_FREQ != FREQ_3906) && (SET_FREQ != FREQ_7751) && (SET_FREQ != FREQ_15625))
#error invalid frequency
#endif

#define IOC_MIN 2.0

// @TODO check IOC voltages in preprocessor using decimals

//#if((uint8_t(3.2 % 4) == 1 ))
//#error IOC_MIN < S1_V < BOARD_V
//#endif

#endif