// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.


#include <xc.h>
#include <stdint.h>
#include <stdbool.h>
#include "pwm.h"
#include "adc.h"
#include "common.h"
#include "modulate.h"
#include "general_io.h"
#include "demo_board.h"
#include "soft_start_stop.h"
#include "bsp.h" // LED defintions


/**
 * TMR1 calculations that provide the set amount of delay as defined in common.h
 * TMR1 is only as accurate as its clock source. The internal RC oscillator is 5%
 * error. If more accuracte timings are desired, place a crystal between the TOSC pins
 *
 * @see TMR1_TIMER_SET
 */
#define TMR1_PRESCALER      1
#define TMR1_CLK_TICK         (1.0 / ((_XTAL_FREQ / 4.0)* TMR1_PRESCALER))  // Fosc/4
#define TMR1_ACCU_WIDTH      ((2^16))
// Calculates number of increments until TIMER_SET is met
#define TMR1_COUNT       (uint16_t)((TMR1_TIMER_SET * .001) / TMR1_CLK_TICK)
// What to fill accumulator after each rollover
#define TMR1_INIT        (uint16_t)(TMR1_ACCU_WIDTH - TMR1_COUNT)

/**
 * Holds modulation data
 */
typedef struct {
    uint16_t phase; // Present phase offset
    uint16_t deltaPhase; // Phase increment (frequency)
    uint8_t voltage; // pulse width (voltage)
} MOD;

MOD mod;

/**
 * These bits are set in the ISR and are to be checked for in the main line code
 */
typedef struct {
    bool doConversion : 1;
    bool stepSoftStart : 1;
    bool stepSoftStop : 1;
    bool toggleMotorLED : 1;
} INTERRUPTS;


INTERRUPTS statusFlags;
uint16_t CurrentTripPoint;


static void SampleSpeed(uint8_t *voltage, uint16_t *deltaPhase);
static void SampleCurrentPinch(uint16_t *CurrentTripPoint);
static bool CheckCurrentSense(uint16_t CurrentTripPoint);
static MOTOR_DIR getMotorDirection(void);
static uint16_t GetAbsSpeed(void);

void SetupBoard() {
    SetupPWM(); // PWM and derived CLC used for drive
    SetupGeneralIO(); // Isolated and Non-isolated inputs
    SetupADC(); // Setup the ADC for the multiplexed inputs

    CurrentTripPoint = CURRENT_TRIP_ADC; // Before any button is pressed, this is the default setting


    /**
     * TMR1 Interrupt for keeping a rough estimate of time for
     * general purposes
     */
    TMR1H = TMR1L = 0; // Reset time registers
    T1CONbits.TMR1CS = 0b00; // FOSC/4
    T1CONbits.T1CKPS = 0b00; // 1:1 prescale
    PIR1bits.TMR1IF = 0;
    PIE1bits.TMR1IE = 1;
    TMR1 = TMR1_INIT; // Init so that interrupt is every TMR1_TIMER_SET
    T1CONbits.TMR1ON = 1; // Start TMR1

    /**
     * TMR0 for Blinking LEDs. Rolls over every 65ms
     * Does NOT cause jumpt to ISR. Simply check the flag
     */
    OPTION_REGbits.PSA = 0; //assign prescaler value of Option_Reg.PS bits
    OPTION_REGbits.TMR0CS = 0; //Fosc/4 as clock source
    OPTION_REGbits.PS = 0b111; //1:256 ratio
    INTCONbits.TMR0IE = 0; // Disable TMR0 interrupt
}

/**
 * Blink the motor LED in proportion to the motor voltage
 *
 * Play around with the tick and division numbers to adjust
 * depending on how long main line code takes 
 */
void BlinkMotor() {
    static uint8_t tick;
    static uint16_t count;

//    if (INTCONbits.TMR0IF) { // Is set upon TMR0 rollover
//        INTCONbits.TMR0IF = 0; // Clear flag
//        tick++;
//    }
    if (count++ == 900) { // Adjust this number as user sees fit
        MOTOR_LED_OUT ^= 1;
        count = mod.voltage;
        count = count << 2; // Multiply so small changes in ModVoltage are amplified
    }
}

bool isSW1() {
    return getSW1();
}

bool isSW2() {
    return getSW2();
}

bool isAUX1() {
    return getAUX1();
}

bool isAUX2() {
    return getAUX2();
}

bool isAUX3() {
    return getAUX3();
}

bool isAUX4() {
    return getAUX4();
}

/*
 * Every start of the motor will perform a soft start
 */
void MotorStart() {
    int16_t speedADC;

    speedADC = GetAbsSpeed(); // 0 < speed < 512 (both directions)
    MotorStop(false); // Stop the motor if it is running
    EnablePWMOut();
    BeginSoftStart(speedADC, getMotorDirection()); // Gets the Speed Wiper and init incremental values
}

/**
 *  Turn the motor OFF by either braking or slowly winding down via
 *  SoftStop
 */
void MotorStop(bool softStop) {
    if (softStop) {
        int16_t speedADC = GetAbsSpeed(); // 0 < speed < 512 (both directions)
        BeginSoftStop(speedADC, getMotorDirection());
    } else {
        // Don't SoftStop...simply stop driving the motor or brake it
        mod.deltaPhase = 0;
        mod.phase = 0;
        mod.voltage = 0;
#ifdef MOTOR_BRAKE
        /*
         * If braking, drive the CLCs HIGH so that the current in the motor
         * windings work against its present inertia.
         */
        IdlePWM(); // Drive PWM LOW
        CLCDrive(true);
#else
        DisablePWMOut(); // Drive all PWM pins LOW
        CLCDrive(false); // Drive all CLC pins LOW
#endif
    }
}

/**
 * Performs the SoftStart
 *
 * @param mod
 * @return boolean indicating when finished with all iterations
 */
bool SoftStart() {
    bool finished = false;

    // Check the interval flag before commencing
    if (statusFlags.stepSoftStart) {
        statusFlags.stepSoftStart = false; // Clear until next interval delay is reached
        finished = InSoftStart(&mod.voltage, &mod.deltaPhase);
        LoadModulateBuffer(mod.voltage); // Load next values
    }
    return finished;
}

/**
 * Performs the SoftStop
 *
 * @param mod
 * @return boolean indicating when finished with all iterations
 */
bool SoftStop() {
    bool finished = false;

    // Check the interval flag before commencing
    if (statusFlags.stepSoftStop) {
        statusFlags.stepSoftStop = false; // Clear until next interval delay is reached
        finished = InSoftStop(&mod.voltage, &mod.deltaPhase);
        LoadModulateBuffer(mod.voltage); // Load next values
    }
    return finished;
}

void MeasSpeed() {
    static uint8_t oldVolts;

    if (statusFlags.doConversion) {
        SampleSpeed(&mod.voltage, &mod.deltaPhase); // Get new V/F profile

        if (mod.voltage != oldVolts) // Don't calculate new values if voltage is the smae
            LoadModulateBuffer(mod.voltage); // Load new profile into buffer

        oldVolts = mod.voltage; // Save previous
        statusFlags.doConversion = false;
    }
}

void MeasCurrentPinch() {
    SampleCurrentPinch(&CurrentTripPoint);
}

bool IsOverCurrent() {
    return CheckCurrentSense(CurrentTripPoint);
}

void ReadSharedAuxInputs() {
    SampleSharedAuxInputs();
}

void ReadSharedButtons() {
    SampleSharedButtons();
}

void ReadDedicatedAuxInput() {
    SampleDedicatedAuxInput();
}

static void SampleSpeed(uint8_t *voltage, uint16_t *deltaPhase) {
    uint16_t SpeedADC = 0;
    MOTOR_DIR dir = 0;
    uint8_t amplitude = 0;
    uint16_t freq;

    // Take ADC sample of speed POT

    dir = getMotorDirection();
    SpeedADC = GetAbsSpeed();

    // The amplitude and frequency of the resultant sine wave have a 1:1 relationship
    // from 0Hz -> 60Hz (or whatever Optimal Motor Hz is). From then onwards, Amplitude
    // is max while frequency increases
    freq = CalcPhase(SpeedADC);
    amplitude = CalcAmplitude(freq, SpeedADC);

    if (dir == REVERSE)
        freq = -freq; // Reverse directions by negating deltaPhase

    // ModVoltage and DeltaPhase are used in the modulate routine
    *voltage = amplitude;
    *deltaPhase = freq;

}

static void SampleCurrentPinch(uint16_t * CurrentTripPoint) {
    // Constantly taking current measurment
    *CurrentTripPoint = GetCurrentPinch(); // Read the ADC
    *CurrentTripPoint >> 1; // Divide by 2; @todo Current Trip range: 0 < T < 2.5V
    *CurrentTripPoint *= CURRENT_TRIP_AVG; // Shift by number of ADC samples to compare to

}

/**
 * This takes an ADC measurement of the Current Sense RC circuit
 *
 * Instead of taking a single measurment and then comparing to the trip level,
 * this method will provide a running summation. It averages 32 samples
 */
static bool CheckCurrentSense(uint16_t CurrentTripPoint) {
    static uint16_t AvgCurrent = 0;
    uint16_t Current; // Present reading of the motor current

    Current = GetCurrentSense(); // Read the 10-bit ADC reading from the low-pass filter
    Current = Current & 0x03FC; // Use only the top 8 bits due to the noise floor

    // Subtract the current average from the past and then add the present 8-bit ADC result
    AvgCurrent = AvgCurrent - (AvgCurrent >> 5) + Current;

    if (AvgCurrent > CurrentTripPoint) {
        return true; // Over current detected
    } else
        return false; // normal conditions
}

static uint16_t GetAbsSpeed() {
    int16_t speedADC;

    speedADC = (GetSpeed() - 512);

    if (speedADC < 0)
        return -speedADC; // Negate
    else
        return (uint16_t) speedADC;
}

static MOTOR_DIR getMotorDirection() {
    int16_t speedADC;

    //Save what direction the motor is in
    /*
     *		          ADC Result Range
    | -512 ******************** -10 ******* 10 ******************** 512 |
    |         Reverse         |      Halt      |          Forward       |
    |*******************************************************************|
     */
    speedADC = (GetSpeed() - 512);
    if (speedADC < 0) { //
        return REVERSE;
    } else
        return FORWARD;
}


/*************************************************************************/
/*************************************************************************/
/*        Called from within ISR - exit as soon as possible              */

/*************************************************************************/
void TMR2ISR() {
    // Write the new values
    WritePWMValues();
    // Load the next values
    mod.phase += mod.deltaPhase; // Add phase difference to adjust modulation angle
    ModulatePWM(mod.phase); // mod.voltage is not needed since it is pre-calculated in MeasSpeed();
}

void TMR1Inc() {
    static uint8_t samplePOTs;
    static uint8_t softStart;
    static uint8_t softStop;


    // Perform an ADC conversion and calculate the V/Hz profile
    if (++samplePOTs == SAMPLE_POT_RATE) {
        statusFlags.doConversion = true;
        samplePOTs = 0;
    }

    // Step through softStart
    if (++softStart == SOFT_START_DELAY) {
        statusFlags.stepSoftStart = true;
        softStart = 0;
    }

    // Step through softStop
    if (++softStop == SOFT_STOP_DELAY) {
        statusFlags.stepSoftStop = true;
        softStop = 0;
    }

    TMR1 = TMR1_INIT; // Reload precalculated TMR1 value
}
/*************************************************************************/
/*************************************************************************/
/*************************************************************************/