// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.

/**
 * @file isolated_io.c
 */
#include <xc.h>
#include <stdint.h> // uint8_t, uint16_t, etc typdefs
#include <stdbool.h>
#include "common.h"
#include "general_io.h"
#include "bsp.h"
#include "adc.h"

typedef struct {
    bool sw1; // Shared
    bool sw2; // Shared
    bool aux1; // Dedicated
    bool aux2; // Shared
    bool aux3; // Shared
    bool aux4; // Shared
    uint16_t speed; // Dedicated
    uint16_t currentLimit; // Dedicated
    uint16_t currentSense; // Dedicated
} INPUTS;

typedef struct{
    bool aux5;
}OUTPUTS;

typedef struct{
    INPUTS inputs;
    OUTPUTS outputs;
}IO;

IO io;

static void SetupIsolatedIO(void);
static void SetupIO(void);

void SetupGeneralIO() {
    SetupIsolatedIO(); // Isolated IO
    SetupIO(); // Non-Isolated IO


    /**
     * Interrupt-On-Change for the following:
     *
     * 2x  push buttons
     * 3x  multiplexed Aux inputs
     * 1x  RF Data in
     *
     * Rising edge detected implies an action. All items normally
     * kept at GND level
     */
    IOCAF = IOCBF = 0; // Clear any existing IOC flags
    IOCAPbits.IOCAP4 = 1;
    IOCAPbits.IOCAP3 = 1;
    IOCBPbits.IOCBP5 = 1;

    INTCONbits.IOCIF = 0; // Clear flag
    INTCONbits.IOCIE = 1; // Enable bit
}

/**
 * IMPORTANT! All of the IOC bits will be reset once they are read!
 *
 */


bool getSW1(){
    bool result;

    result = io.inputs.sw1;
    io.inputs.sw1 = false; // clear
    return result;
}
bool getSW2(){
    bool result;

    result = io.inputs.sw2;
    io.inputs.sw2 = false; // Clear
    return result;
}

bool getAUX1(){
    bool result;

    result = io.inputs.aux1;
    io.inputs.aux1 = false; // Clear
    return result;
}

bool getAUX2(){
    bool result;

    result = io.inputs.aux2;
    io.inputs.aux2 = false; // Clear
    return result;
}

bool getAUX3(){
    bool result;

    result = io.inputs.aux3;
    io.inputs.aux3 = false; // Clear
    return result;
}

bool getAUX4(){
    bool result;

    result = io.inputs.aux4;
    io.inputs.aux4 = false; // Clear
    return result;
}

/**
 * Enter here when one of the two switch buttons was pressed
 */
void SampleSharedButtons() {
    uint16_t ADCVal; // Holds the ADC reading of the buttons

    // Switch 1 was pressed
    INTCONbits.IOCIE = 0; // Disable future button interrupts while reading pin
    if (true) {
        BUTTONS_ANS = 1; // Change to analog to take an ADC reading
        __delay_ms(25); //debounce
        ADCVal = GetButtons(); // Read the analog voltage on the push buttons pin
        if (ADCVal > IOC_ADC) { // Button is normally at GND when not pressed
            if (ADCVal > S_MID_ADC) {
                // Switch 1 was pressed
                io.inputs.sw1 = true;
            } else {
                // Switch 2 was pressed
                io.inputs.sw2 = true;
            }
        }
    }
    BUTTONS_ANS = 0; // Change back to digital for IOC
    INTCONbits.IOCIE = 1; // Re-enable button IOC
}

void SampleSharedAuxInputs() {
    uint16_t ADCVal; // Holds the ADC reading 

    GENERAL_ANS = 1; // Change to analog
    ADCVal = GetAuxInputs(); // Read the analog voltage on the push buttons pin
    if (ADCVal > IOC_ADC) {
        if (AUX2_ADC < ADCVal && ADCVal < AUX3_ADC) {
            io.inputs.aux2 = true;
        } else if (AUX3_ADC < ADCVal && ADCVal < AUX4_ADC) {
            io.inputs.aux3 = true;
        } else {
            io.inputs.aux4 = true;
        }
    }
    GENERAL_ANS = 0; // Change back to digital I/O
}

void SampleDedicatedAuxInput(){
    if(DEDICATED_IN_IN)
        io.inputs.aux1 = false;
    else
        io.inputs.aux1 = true;
}

static void SetDedicatedAuxOutput(bool set){
    DEDICATED_OUT = set;
}

static void SetupIO() {
    // Current Sense
    CURRENT_SENSE_DIR = 1; // Current Sense input
    CURRENT_SENSE_ANS = 1; // Current Sense is analog

    // LEDs
    MOTOR_LED_DIR = 0; // Motor Indicator
    TRIP_LED_DIR = 0; // Board PWR indicator
    MOTOR_LED_OUT = 0;
    TRIP_LED_OUT = 0;
}

static void SetupIsolatedIO() {
    // 1x8 Header
    GENERAL_DIR = 1; // Isolated Digital inputs are inputs
    GENERAL_ANS = 0; // Digital for IOC...After interrupt change to analog to figure out what input was active
    DEDICATED_IN_DIR = 1; // RF/MCLR  (dedicated digital input)

    // Buttons
    BUTTONS_DIR = 1; // Two switches input
    BUTTONS_ANS = 0; // Digital for IOC...After interrupt change to analog to figure out what button was pressed

    //POTs
    SPEED_POT_DIR = 1; // Speed POT input
    CURRENT_POT_DIR = 1; // Current Pinch POT input
    SPEED_POT_ANS = 1; // Speed POT is analog
    CURRENT_POT_ANS = 1; // Current Pinch POT is analog

}



