// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.

/**
 * AC Induction Motor Controller Board revB
 *
 *
 * VF controlled from MODULATION_MIN to MODULATION_MAX (Hz)
 * PWM switching frequency at 3.9KHz and optionaly  at 7..8KHz and 15.6KHz
 *
 *
 * This simple controller board is based on the PIC16F1509. It provides a 3-phase
 * FET/IGBT drive with 3 PWM signals and their complements. Each phase is 90 deg
 * from the other, except betweens phases 3 -> 1. The PIC can also drive a tpical
 * 3-phase motor as well as a single-phase motor via a simple #define in modulate.c
 *
 * The Sine Wave is constructed by TMR2 PWM. A modulate routine is called every overflow
 * that uses a lookup table to construct the waveform. Two variables control the function:
 *
 * DeltaPhase - Determines the resultant sine wave frequency
 * ModVoltate - Determines the resultant sine wave magnitude
 *
 * 
 *
 * Operation:
 *
 * Pressing one the buttons will toggle the motor ON/OFF. The motor will Soft-Start
 * to the optimal 60Hz full speed and then fall-back to whatever setting the Speed POT is at.
 * A single POT will regulate the speed of the motor (MOD_MIN (hz) ->MOD_MAX (Hz)).
 * Upon turn-off, the motor will SoftStop or brake (optinally) or coast to a stop
 * 
 *
 * I/O:
 *
 * All user interface such as the POTs, buttons, and auxillary I/O are isolated for
 * safety purposes. This requires the use of optocouplers. The rest of the circuitry
 * is considered a hazard. All ground circuitry is floating above earth ground, therefore
 * be careful when attaching a non-isolated bench osilloscope.
 *
 *
 * The following table is the I/O mapping
 *
 * Pin	I/O	Type            Purpose             Notes
 *
 * RA5	OUT	DIGITAL         PWR LED
 * RA4	IN	BOTH            Push Buttons	2x Shared In
 * RA3	IN	DIGITAL         RF Data Stream	Dedicated In
 * RC5	OUT	DIGITAL         Motor
 * RC4	OUT	DIGITAL         Garage Light
 * RC3	OUT	DIGITAL         Motor
 * RC6	IN	ANALOG          Current Sense
 * RC7	OUT	DIGITAL     	Motor LED
 * RB7	OUT	DIGITAL         Motor
 * RB6	OUT	DIGITAL         I2C Clock	Unused
 * RB5	IN	BOTH            Auxiliary Input 3x Shared In
 * RB4	BOTH	DIGITAL         I2C Data	Unused
 * RC2	IN	ANALOG          Motor Speed POT	Dedicated In
 * RC1	OUT	DIGITAL         Motor
 * RC0	OUT	DIGITAL         Motor
 * RA2	OUT	DIGITAL         Motor
 * RA1	FREE	ANALOG          FREE	FREE
 * RA0	IN	ANALOG          Current Pinch	Dedicated In
 *
 * Some of the Inputs are shared. This is because an ADC can be used to multiplex
 * inputs without the overhead of each of their own pins. Some inputs, such as RF
 * data, must have its own dedicated input whilst others do not. The door trip
 * sensors only provide a boolean input which can easily be multiplexed.
 *
 * Please note that any #defines or regualr in-line code with constants of type decimal
 * (1.0) should be left as-is or replaced with a decimal. This is to ensure floating point
 * math is performed.
 *
 * @since revA
 * @author Justin Bauer
 * @date 4.5.2013
 *
 *
 */

#include <xc.h> // compiler includes
#include <stdint.h> // uint8_t, uint16_t, etc typdefs
#include <stdbool.h> // boolean types
#include "bsp.h"
#include "common.h"
#include "demo_board.h"

#if defined (_16LF1509) || defined(_16F1509)
#if defined(__DEBUG)
#pragma config CP=OFF, PWRTE=OFF, MCLRE=ON
#else
#pragma config CP=ON, PWRTE=ON, MCLRE=OFF
#endif

#pragma config WRT=OFF
#pragma config BOREN=OFF, IESO=OFF, FOSC=INTOSC, FCMEN=OFF, WDTE=OFF
#pragma config CLKOUTEN=OFF, LVP=OFF, STVREN=ON, BORV=LO, LPBOR=OFF
#else
#error unknown processor
#endif

/**
 * These bits are set in the ISR and are to be checked for in the main line code
 */
typedef struct {
    bool buttonsIOC : 1;
    bool auxSharedIOC : 1;
    bool auxDedicatedIOC : 1;
    bool TMR1OVF : 1;
} INTERRUPTS;

typedef enum {
    INIT,
    IDLE,
    MOTOR_STARTING,
    MOTOR_ON,
    MOTOR_STOPPING,
} MAIN_STATES;


INTERRUPTS flags;
static void Init(void);

/**
 * Main program
 *
 *   Module      |     Usage
 * *************************************
 *   TMR2            PWM timing
 *   TMR1            All timing related tasks (softstart/stop)
 *   IOC             Push-Buttons AND 3 external auxillary inputs
 *   ADC             @ADC_OPTIONS
 *   CLC             Motor Drive
 *   PWM             Motor Drive
 */

void main(void) {
    bool finished;
    MAIN_STATES mainState;

    /**
     * Main Loop
     *
     * Take a sample of the SPEED_POT and CURRENT_PINCH_POT at a rate of REFRESH_RATE
     */
    mainState = INIT;
    while (1) {
        /**
         * Tasks
         *
         * 1. Get speed input and adjust V/F profile when NOT in Soft-Start
         * 2. Perform Soft-Start for short duration
         * 3. Blink LED in relation to the motor speed
         * 4. Take current sense measurment
         *
         * Much of the CPU is spent in the modulation function, else it is taking ADC
         * measurement of the POTs. It is vital to ensure that the modulation function
         * takes LESS time to complete than the PWM period!!
         */

        switch (mainState) {
            case INIT:
                Init();
                MotorStop(false);
                mainState = IDLE;
                break;
            case IDLE:
                if (isSW1()) {
                    // SW1 pressed
                    MotorStart(); // Motor ON
                    mainState = MOTOR_STARTING;
                } else if (isSW2()) {
                    // SW2 pressed
					MotorStart(); // Motor ON
                    mainState = MOTOR_STARTING;
                    //                    MotorStart(); // Motor ON
                    //                    SoftStart(); // Set Motor to 0Hz
                    //                    SoftStart(); // Set Motor to MODULATION_MIN
                    //                    SoftStart(); // Start now
                    //                    MOTOR_LED_OUT = 1;
                }
                break;
            case MOTOR_STARTING:
                finished = SoftStart();
                BlinkMotor();
                if (finished == true) {
                    finished = false;
                    mainState = MOTOR_ON; // SoftStart has finished
                }
                break;
            case MOTOR_ON:
                MeasSpeed();
                MeasCurrentPinch();
                BlinkMotor();
                if (IsOverCurrent()) {
                    MotorStop(false); // Stop motor if overcurrent
                    // Blocking code
                    for (uint8_t i = 0; i != 5; i++) {
                        TRIP_LED_OUT ^= 1;
                        __delay_ms(300);
                    }
#if(REV_NUM != 3)
                    TRIP_LED_OUT = 1;
#else
                    TRIP_LED_OUT = 0;
#endif
                    mainState = MOTOR_STOPPING;
                }
                if (isSW1()) {
                    // SW1 pressed
                    MotorStop(true);
                    mainState = MOTOR_STOPPING; // Stop the Motor
                } else if (isSW2()) {
                    // SW2 pressed
					MotorStop(true);
                    mainState = MOTOR_STOPPING; // Stop the Motor
                    NOP();
                }

                if (isAUX1()) {
                    // AUX1 input ACTIVE
                } else if (isAUX2()) {
                    // AUX2 input ACTIVE
                } else if (isAUX3()) {
                    // AUX3 input ACTIVE
                } else if (isAUX4()) {
                    // AUX4 input ACTIVE
                }
                break;
            case MOTOR_STOPPING:
                finished = SoftStop();
                BlinkMotor();
                if (finished == true) {
                    finished = false;
                    MotorStop(false); // Keep motor at 50% duty cycle (not turning - idle)
                    MOTOR_LED_OUT = 0;
                    mainState = IDLE; // SoftStop has finished
                }
                break;
            default:
                break;

        }

        /**
         * Now Service any interrupts that may have occured
         *
         * @note Do not service this inside of ISR in order to preserve
         *       the sine wave accuracy
         */
        if (flags.TMR1OVF) {
            // Instead, perform this inside of ISR
            // TMR1Inc(); // Keept time
            // flags.TMR1OVF = false;
        }
        if (flags.auxSharedIOC) {
            ReadSharedAuxInputs();
            flags.auxSharedIOC = false;
        }
        if (flags.auxDedicatedIOC) {
            ReadDedicatedAuxInput();
            flags.auxDedicatedIOC = false;
        }
        if (flags.buttonsIOC) {
            ReadSharedButtons();
            flags.buttonsIOC = false;
        }
    }

}

static void Init() {
    OSCCON = 0x00;
    OSCCONbits.IRCF = IRCF_VAL; // 16 MHz internal RC oscillator

    // General initlization to digital output LOW for low power consumption for unused pins
    TRISA = 0; //output
    TRISB = 0; //output
    TRISC = 0; //output
    ANSELA = 0; //digital
    ANSELB = 0; //digital
    ANSELC = 0; //digital
    SetupBoard();

    // If DEBUG is enabled, PWM4 will be transmitted across the optocoupler
    // on P2. Use this to demonstrate the hi-speed digital output capabilities
#ifdef DEBUG
    CLC4GLS0 = 0x00;
    CLC4GLS1 = 0x00;
    CLC4GLS2 = 0x00;
    CLC4GLS3 = 0x80;
    CLC4SEL0 = 0x00;
    CLC4SEL1 = 0x30;
    CLC4POL = 0x04;
    CLC4CON = 0xC0;
#endif


    //Start up sequence
    for (uint8_t i = START_BLINKS; i--; i != 0) {
        TRIP_LED_OUT ^= 1;
        __delay_ms(START_DELAY);
        MOTOR_LED_OUT ^= 1;
    }
    MOTOR_LED_OUT = 0;
    TRIP_LED_OUT = 0;

#if(REV_NUM != 3)
    PWR_LED_DIR = 0;
    PWR_LED_OUT = 1;
#endif

    INTCONbits.PEIE = 1; // Enable peripheral interrupts (TMR0, TMR2, IOC)
    INTCONbits.GIE = 1; // Enable global interrupts
}

/**
 *              ISR processes modulation interrupts for PWM
 *
 * Do all time consuming tasks outside of ISR EXCEPT for reloading PWM registers
 * and incrementing TMR1. Can optinally move this out of the ISR
 *
 *  Flag   |         Purpose                            |  Interval
 * *************************************************************************
 *  TMR2     Calculate next PWM values                      255us
 *  TMR1     General Purpose timer                         User defined (
 *  IOCxF    Interrupt-on-Change for inputs                User determined
 */
static void interrupt ISR(void) {
    if (PIR1bits.TMR2IF && PIE1bits.TMR1IE) // If TMR2 rolled over
    {
        PIR1bits.TMR2IF = 0; // Clear interrupt flag
        // PWM modulation takes precedence over everything! It must be performed
        // inside of the ISR so that the sine wave timings are accurate
        TMR2ISR(); // Assign PWM values
    }

    /**
     * Interrupt-On-Change
     */
    if (INTCONbits.IOCIF && INTCONbits.IOCIE) { // Enter here for the push buttons
        if (BUTTONS_IOC_FLAG) {
            // Buttons pressed
            flags.buttonsIOC = true;
        } else if (GENERAL_IOC_FLAG) {
            // Auxillary shared input
            flags.auxSharedIOC = true;
        } else if (DEDICATED_IN_IOC_FLAG) {
            // Auxillary dedicated input
            flags.auxDedicatedIOC = true;
        }
        IOCAF = 0; // Must do this for IOCIF to clear
        IOCBF = 0; // Must do this for IOCIF to clear
    }


    if (PIR1bits.TMR1IF && PIE1bits.TMR1IE) {
        PIR1bits.TMR1IF = 0; // Clear flag
        // flags.TMR1OVF = true;
        TMR1Inc();
    }
}

