// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.


/**
 *@file modulate.c
 * 
 */

#include <xc.h> //@todo debug
#include <stdint.h> // uint8_t, uint16_t, etc typdefs
#include "modulate.h"
#include "common.h"
#include "pwm.h"

// Introduce a 50% offset into the sine value.
// If volts is zero, then the PWM width will be HALF_DUTY 
// The smaller the volts value, the smaller the height of the Sine wave. When Volts = 0, output is simply Vdd/2

/**
 * Best performance is seen when all numbers are powers of 2
 *
 * TABLE_DIV :   INT8_MAX * AMPLITUDE_MAX / TABLE_DIV == HALF_DUTY
 * HALF_DUTY : (2 ^ PWM resolution) / 2
 * PHASE_DIV : ((MODULATION_MAX / (Fpwm / 2 ^ UINT16_MAX)) / 2^PHASE_DIV) == 512
 */

#if(SET_FREQ == FREQ_3906)
#define TABLE_DIV 32 
#define HALF_DUTY 512 
#define PHASE_DIV  2 
#elif(SET_FREQ == FREQ_7751)
#define TABLE_DIV 64 
#define HALF_DUTY 256 
#define PHASE_DIV 1 
#elif(SET_FREQ == FREQ_15625)
#define TABLE_DIV 32
#define HALF_DUTY 512
#define PHASE_DIV 0
#endif


#define NINETY_DEG          0x4000 //16-bit number divided by 4 to get 90
#define ONE_TWENTY_DEG      0x5555 // 16-bit number divided by 3 to get 120

#define SINE_BUFFER_SIZE    64 // Should equal total number of values in sine table


// Place in RAM for quick access
uint16_t sineBuffer1[SINE_BUFFER_SIZE];

static void FillPhase(uint16_t *phase, uint8_t index);

#if MOTOR_TYPE == SINGLE_PHASE
// This is a 64-entry table containing sine content.
// The values are scaled to be within the limits of an 8 bit signed value.
const int8_t sineTable[SINE_BUFFER_SIZE] = {0, 12, 25, 37, 49, 60, 71, 81, 90, 98, 106, 112,
    117, 122, 125, 126, 127, 126, 125, 122, 117, 112, 106, 98, 90, 81, 71, 60, 49,
    37, 25, 12, 0, -12, -25, -37, -49, -60, -71, -81, -90, -98, -106, -112, -117,
    -122, -125, -126, -127, -126, -125, -122, -117, -112, -106, -98, -90, -81,
    -71, -60, -49, -37, -25, -12};

#define OFFSET NINETY_DEG

#elif MOTOR_TYPE == THREE_PHASE
// Please see the "Sine with 3rd Harmonics.xls" sheet for the derivation of the following values
const int8_t sineTable[SINE_BUFFER_SIZE] = {0, 21, 41, 60, 77, 92, 104, 113, 119, 123, 125,
    125, 125, 124, 122, 121, 121, 121, 122, 124, 125, 125, 125, 123, 119, 113, 104, 92, 77,
    60, 41, 21, 0, -21, -41, -60, -77, -92, -104, -113, -119, -123, -125, -125, -125, -124,
    -122, -121, -121, -121, -122, -124, -125, -125, -125, -123, -119, -113, -104, -92, -77,
    -60, -41, -21
};

#define OFFSET ONE_TWENTY_DEG

#endif

/**
 * Instead of calculating the sine wave values after every TMR2 overflow, this routine calculates
 * the new values out of the interrupt service routine and saves them in RAM where they can be
 * quickly accessed.
 *
 * Since a sine wave can be constructed by manipulating a single quarter of the wave, you can
 * optionally store the first quarter wave values in RAM and then determine the 3/4 rest of the
 * sine wave from that. Note however that this will result in more time taken inside of the ISR,
 * but less time inside this routine
 *
 *
 * SoftStart/Stop must not be shorter than it takes for this routine to complete!
 * @warning This routine may take time on the order of milliseconds (ms)
 * @param volts
 */
void LoadModulateBuffer(uint8_t volts) {
    int16_t temp;

    if (volts > AMPLITUDE_MAX) {
        volts = AMPLITUDE_MAX; //Max allowed 
    }
 //   LATBbits.LATB7 = 1; CLC3CON  = 0x00;
    for (uint8_t i = 0; i < SINE_BUFFER_SIZE; i++) {
        /*
         * ((AMPLITUDE_MAX * SineTableMAX)/ RIGHT_SHIFT) must ~= HALF_DUTY
         *
         */
        temp = (sineTable[i] * volts) / TABLE_DIV; // Multply by volts and then divide by TABLE_DIV to scale
        // (127 * AMPLITUDE_MAX) / (2 ^ shift_right_num) == HALF_DUTY
        sineBuffer1[i] = (uint16_t) (temp) + HALF_DUTY;
    }
 //   LATBbits.LATB7 = 0; CLC3CON  = 0x00;
}
/**
 * Use the pre-calculated values in the sineBuffer to quickly load the PWM registers
 *
 * @param angle unsigned 16-bit int with phase information
 * @param phase1 pwm1 pulse width
 * @param phase2 pwm2 pulse width
 * @param phase3 pwm3 pulse width
 */
void Modulate(uint16_t angle, uint16_t *phase1, uint16_t *phase2, uint16_t *phase3) {
    // LATCbits.LATC4 = 1;

    union {
        uint16_t ui;
        uint8_t uc[2];
    } phase_index;

    uint8_t index;

    phase_index.ui = angle;
    // Get the lookup value using upper 6 bits of the MSB phase index (64-byte table)
    index = phase_index.uc[1] >> 2;
    // Assign the delta_phase value to create frequency
    *phase1 = sineBuffer1[index];
    // FillPhase(phase1, index);

    // Calculate the duty cycle for Phase 2
    phase_index.ui += OFFSET;
    index = phase_index.uc[1] >> 2;
    *phase2 = sineBuffer1[index];
    //   FillPhase(phase2, index);

    // Calculate the duty cycle for Phase 3
    phase_index.ui += OFFSET;
    index = phase_index.uc[1] >> 2;
    *phase3 = sineBuffer1[index];
    //   FillPhase(phase3, index);


    // LATCbits.LATC4 = 0;

}

/**
 * Optinal routine to calculate the sine wave while inside of the TMR2 ISR instead of
 * calculating the entire sineTable when the volts paramter changes. This method will take
 * longer in the ISR and hence provide a slower achievable PWM frequency, but will save
 * space in the sineBuffer and time calculating all 64 values and do just 16 instead (quarter wave).
 *
 * @warning Must subtract out HALF_DUTY before negating from sineTable. Need to midify Loadmodulatebuffer
 * 
 * @param phase
 * @param index
 */
static void FillPhase(uint16_t *phase, uint8_t index) {
    if (index <= 15) { // 1st quarter
        *phase = sineBuffer1[index] + HALF_DUTY;
    } else if ((index > 15) && (index < 31)) { // 2nd quarter
        *phase = sineBuffer1[SINE_BUFFER_SIZE - (index - 16)] + HALF_DUTY;
    } else if ((index >= 31) && (index < 47)) { // 3rd quarter
        *phase = -sineBuffer1[index - 32] + HALF_DUTY;
    } else { // 4th quarter
        *phase = -sineBuffer1[SINE_BUFFER_SIZE - (index - 48)] + HALF_DUTY;
    }
}

/**
 * Each shift left will double the sine wave frequency and shift right will halve it.
 *
 * (MODULATION_MAX / (Fpwm / 2 ^ 16)) = deltaPhase
 *
 * example: (120Hz / (7812Hz / 2 ^ 16)) = 1006
 *
 * 1006 / 2 ~=
 *
 * example: SpeedADC << 1 yields a max frequency of 60Hz (assuming 3.9KHz PWM)
 *          SpeedADC << 2 yields a max frequency of 120Hz (assuming 3.9KHz PWM)
 *
 *          SpeedADC << 1 yields a max frequency of 120Hz (assuming 7.8KHz PWM)
 *          SpeedADC << 2 yields a max frequency of 240Hz (assuming 7.8KHz PWM)
 *
 * @param ADCResult  10-bit Value   0 < ADCResult < 512
 */
uint16_t CalcPhase(uint16_t ADCResult) {
    uint16_t deltaPhase;

    deltaPhase = ADCResult << PHASE_DIV;
    if (deltaPhase < DELTA_MIN) {
        deltaPhase = DELTA_MIN;
    }

    return deltaPhase;
}

/**
 * Calculates the amplitude of the resultant sine wave.
 *
 *
 * @param phase
 * @param ADCResult 10-bit Value   0 < ADCResult < 512
 * @return
 */
uint8_t CalcAmplitude(uint16_t phase, uint16_t ADCResult) {
    uint8_t Amplitude;

    // If the frequency is above the ideal, then voltage must be at its maximum
    if (phase >= DELTA_IDEAL) {
        Amplitude = AMPLITUDE_MAX;
    }// If the frequency is below the ideal, then voltage must be at its minimum allowed or Zero
    else if (phase <= DELTA_MIN) {
        if (ADCResult < ADC_STOP_POS) // Check if should stop modulation
            Amplitude = 0;
        else
            Amplitude = AMPLITUDE_MIN;
    }// Else the motor is somewhere inbetween the min and ideal fequency
    else {
        // When Frequency == MODULATION_IDEAL, start to decrement voltage
        // ADCResult should be adjusted from its maximum when this happens
        Amplitude = (uint8_t) (ADCResult >> 1); // Divide to scale to within AMPLITUDE_MAX @ MODULATOIN_IDEAL(512 / 2 ) = 128
    }
    return (uint8_t) Amplitude;
}