// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.

/**
 * @file pwm.c
 * 
 * 6 digital outputs for IGBT/FET driver
 *
 * 3 CLCs (CLC2, 3, 1)
 * 3 PWMs (PWM1, 2, 4)
 *
 * CLC2 = nPWM1
 * CLC3 = nPWM2
 * CLC1 = nPWM4
 */


#include <xc.h>
#include <stdint.h> // uint8_t, uint16_t, etc typdefs
#include <stdbool.h>

#include "pwm.h"
#include "bsp.h"
#include "common.h"
#include "modulate.h"

// 10-bit resolution achieved with PR2 set to 255
// Use this in conjunction with TMR2 prescale to adjust PWM switching frequency
#if(SET_FREQ == FREQ_3906)
#define T2CON_PRESCALE 4
#define PR2_VAL 255
#elif(SET_FREQ == FREQ_7751)
#define T2CON_PRESCALE 4
#define PR2_VAL 128
#elif(SET_FREQ == FREQ_15625)
#define T2CON_PRESCALE 1
#define PR2_VAL 255
#endif



#if (T2CON_PRESCALE == 1)
#define TMR2_PRESCALE 0b00
#elif (T2CON_PRESCALE == 4)
#define TMR2_PRESCALE   0b01
#elif (T2CON_PRESCALE == 16)
#define TMR2_PRESCALE 0b10
#elif (T2CON_PRESCALE == 64)
#define TMR2_PRESCALE
#else
#error incorrect T2CON prescaler
#endif

#define PWM_PER ((PR2_VAL + 1) * 4 * (1/(_XTAL_FREQ * 1.0)) * T2CON_PRESCALE) // PWM period
#define PWM_FREQ (1.0/PWM_PER) // PWM Freq


// Declare a structure to hold PWM duty cycles
DUTY pwm;

static void WriteDutyCCP1(uint16_t dutycycle);
static void WriteDutyCCP2(uint16_t dutycycle);
static void WriteDutyCCP3(uint16_t dutycycle);
static void WriteDutyCCP4(uint16_t dutycycle);
static void SetupTMR2(void);
static void SetupCLC(void);

/**
 * Setup the CCPs as PWM single mode
 *
 * NOT using PWM3 module
 *
 */
void SetupPWM() {
    IdlePWM(); // Start OFF until button is pressed
    SetupTMR2(); // TMR2 used in PWM module to generate square wave
    SetupCLC(); // Complement Drive

    PWM1_DIR = 0; // All outputs
    PWM2_DIR = 0;
    PWM4_DIR = 0;

    PWM1CON = 0b11000000; //enable module, output enabled, active HIGH
    PWM2CON = 0b11000000; //enable module, output enabled, active HIGH
    PWM4CON = 0b11000000; //enable module, output enabled, active HIGH
}

void ModulatePWM(uint16_t angle) {
    Modulate(angle, &pwm.phase1, &pwm.phase2, &pwm.phase3); // Perform sine modulation
}

void WritePWMValues() {
    // Write the new duty cycles to the PWM outputs
    // NOT using PWM3 pin
    WriteDutyCCP1(pwm.phase1);
    WriteDutyCCP2(pwm.phase2);
    WriteDutyCCP4(pwm.phase3);
}

void DisablePWMOut(){
    PWM1_OUT = 0;
    PWM2_OUT = 0;
    PWM4_OUT = 0;

    PWM1CONbits.PWM1OE = 0;
    PWM2CONbits.PWM2OE = 0;
    PWM4CONbits.PWM4OE = 0;
}

void EnablePWMOut(){
    PWM1CONbits.PWM1OE= 1;
    PWM2CONbits.PWM2OE = 1;
    PWM4CONbits.PWM4OE = 1;
}

/**
 * Keeps the PWM modules enabled and fills pulse widths to 0
 */
void IdlePWM() {
    PWM1DCH = 0;
    PWM1DCL = 0;
    PWM2DCH = 0;
    PWM2DCL = 0;
    PWM4DCH = 0;
    PWM4DCL = 0;
}

void CLCDrive(bool brake) {
    if (brake) {
        // Drive CLC output gate HIGH to brake
        LC1OUT = 1;
        LC2OUT = 1;
        LC3OUT = 1;
    } else {
        // Drive CLC output gate LOW
        LC1OUT = 0;
        LC2OUT = 0;
        LC3OUT = 0;
    }
}

/**
 * TMR2 for PWM
 *
 * Currently setup to roll-over every 255us,
 * creating a PWM frequency of 3.921KHz
 *
 * @note We are limited in the PWM frequency in however fast the
 * 'modulate' function is performed. The next calculation is always
 * being performed during the PWM period
 *
 * Rollover = ClockSource * PreScaler * PR2
 * 255us = 1 / ((16MHz / 4)) * 4 * 255
 */
static void SetupTMR2() {
    PR2 = PR2_VAL;
    TMR2 = 0; //start OFF
    T2CON = 0b00000000;
    T2CONbits.T2CKPS = TMR2_PRESCALE;
    TMR2IF = 0; //clear interrupt flag
    PIR1bits.TMR2IF = 0; // clear TMR2 flag
    PIE1bits.TMR2IE = 1; // Enable TMR2 interrupts
    TMR2ON = 1; //start TMR2
}

static void SetupCLC() {
#include "clc.inc"   // CLC config tool generated output

    CLC1_DIR = 0; // All outputs
    CLC2_DIR = 0;
    CLC3_DIR = 0;
}

/**
 * Writes to duty cycle registers
 * @param dutycycle
 */
static void WriteDutyCCP1(uint16_t dutycycle) {
    if (dutycycle & 0x0001) PWM1DCLbits.PWM1DCL0 = 1;
    else PWM1DCLbits.PWM1DCL0 = 0;
    dutycycle >>= 1;
    if (dutycycle & 0x0001) PWM1DCLbits.PWM1DCL1 = 1;
    else PWM1DCLbits.PWM1DCL1 = 0;
    dutycycle >>= 1;
    PWM1DCH = (uint8_t) dutycycle;
}

/**
 * Writes to duty cycle registers
 * @param dutycycle
 */
static void WriteDutyCCP2(uint16_t dutycycle) {
    if (dutycycle & 0x0001) PWM2DCLbits.PWM2DCL0 = 1;
    else PWM2DCLbits.PWM2DCL0 = 0;
    dutycycle >>= 1;
    if (dutycycle & 0x0001) PWM2DCLbits.PWM2DCL1 = 1;
    else PWM2DCLbits.PWM2DCL1 = 0;
    dutycycle >>= 1;
    PWM2DCH = (uint8_t) dutycycle;
}

/**
 * Writes to duty cycle registers
 * @param dutycycle
 */
static void WriteDutyCCP4(uint16_t dutycycle) {
    if (dutycycle & 0x0001) PWM4DCLbits.PWM4DCL0 = 1;
    else PWM4DCLbits.PWM4DCL0 = 0;
    dutycycle >>= 1;
    if (dutycycle & 0x0001) PWM4DCLbits.PWM4DCL1 = 1;
    else PWM4DCLbits.PWM4DCL1 = 0;
    dutycycle >>= 1;
    PWM4DCH = (uint8_t) dutycycle;
}


