// MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software,
// and any derivatives created by any person or entity by or on your
// behalf, exclusively with Microchip's products. Microchip and its
// licensors retain all ownership and intellectual property rights in
// the accompanying software and in all derivatives hereto.
//
// This software and any accompanying information is for suggestion
// only. It does not modify Microchip's standard warranty for its
// products. You agree that you are solely responsible for testing the
// software and determining its suitability. Microchip has no obligation
// to modify, test, certify, or support the software.
//
// THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES,
// WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
// IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS
// FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
// MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN
// ANY APPLICATION.
//
// IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
// TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
// LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
// SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS,
// DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE
// SOFTWARE, HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE
// POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT
// ALLOWABLE BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY
// WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF
// ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
//
// MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
// OF THESE TERMS.


/**
 * @file soft_start_stop.c
 */

#include <stdint.h>
#include <stdbool.h>
#include "soft_start_stop.h"
#include "common.h"
#include "modulate.h" // AMPLITUDE_MAX @todo optimize this out


/**
 * Holds all of the properites needed to perform a SoftStart of the motor.
 * SoftStart slowely increments the frequency and voltage in a 1:1 fashion
 * from MIN_FREQ -> MOTOR_IDEAL_FREQ. It then increases the frequency and
 * holds the voltage at maximum from MOTOR_IDEAL_FREQ->SET_FREQ
 */
typedef struct {
    uint8_t incSoftStart; // # of times currently stepped
    float incDeltaPhase;
    float incVoltage; // After every step, preserve the float accuracy
    float addDeltaPhase;
    float addVoltage;
    MOTOR_DIR direction;

} STEP_MODULATION;

typedef union {
    STEP_MODULATION start;
    STEP_MODULATION stop;
} SOFT_START_STOP;



SOFT_START_STOP stopStart;


static void InitSoftStep(STEP_MODULATION *step, uint16_t speed, MOTOR_DIR dir);
static bool DoSoftStart(STEP_MODULATION *start, uint8_t *voltage, uint16_t *deltaPhase);
static bool DoSoftStop(STEP_MODULATION *stop, uint8_t *voltage, uint16_t *deltaPhase);

void BeginSoftStart(uint16_t speed, MOTOR_DIR dir) {
    InitSoftStep(&stopStart.start, speed, dir);
}

bool InSoftStart(uint8_t *voltage, uint16_t *phase) {
    return DoSoftStart(&stopStart.start, voltage, phase);
}

void BeginSoftStop(uint16_t speed, MOTOR_DIR dir) {
    InitSoftStep(&stopStart.stop, speed, dir);
}

bool InSoftStop(uint8_t *voltage, uint16_t *phase) {
    return DoSoftStop(&stopStart.stop, voltage, phase);
}

static void InitSoftStep(STEP_MODULATION *step, uint16_t speed, MOTOR_DIR dir) {
    float setFreq; // Set by the Speed POT wiper
    float stepPhase; // units of Hz

    // Is either = MODULATION_IDEAL or (MODULATION_MIN < X < setFreq)
    float freqAmplitudeMax;
    uint8_t stepsTillMaxVoltage;

    step->direction = dir; // Get current Speed POT position

    /**
     * An input of 512 indicates maximum ending frequency from
     * the SoftStart.
     *
     * This calcuation requires floating Division AND multiplication!!
     *
     * Ex: 50 passes from 20Hz -> 120Hz
     *  + 2Hz increment
     *  + 5 Amplitude increase
     */
    // Increase frequency linearly from MODULATION_MIN to ADC value (or MODULATION_MAX)
    setFreq = (speed / 512.0) * MODULATION_MAX; // In Hz, what frequency to SoftStart to
    if (setFreq < MODULATION_MIN) { // Protect against the minimum freq
        setFreq = MODULATION_MIN;
    }
    stepPhase = (((setFreq * 1.0) - MODULATION_MIN) / SOFT_START_DURATION); // In Hz, how many to increment each iteration
    step->incDeltaPhase = stepPhase / MOD_RES; // This is what is added to the 16-bit num for modulation

    /******************************************************************
     *  Now get the voltage to increment every iteration of SoftStart
     *
     * Increase PWM voltage linearly from MODULATION_MIN to MODULATION_IDEAL
     *****************************************************************/

    if (setFreq > MODULATION_IDEAL) {
        freqAmplitudeMax = MODULATION_IDEAL; // In Hz, where the amplitude will cease to increment
    } else
        freqAmplitudeMax = setFreq;
    // Only difference here from the DeltaPhase calculation is using IDEAL and not MAX modulation
    // Since the Amplitude will maximize at the ideal modulation
    stepsTillMaxVoltage = (uint8_t) ((freqAmplitudeMax - MODULATION_MIN) / stepPhase); // In Hz, number of steps till max voltage
    step->incVoltage = (float) (AMPLITUDE_MAX - AMPLITUDE_MIN) / stepsTillMaxVoltage; // Number of volts to increment per step

    // Reset for new instance of SoftStart
    step->incSoftStart = 0; // Reset count
    step->addDeltaPhase = 0;
    step->addVoltage = 0;
}

static bool DoSoftStop(STEP_MODULATION *stop, uint8_t *voltage, uint16_t *deltaPhase) {
    // Enter here xx times to DECREASE frequency by xxHz from xxHz up to xxHz
    // and DECREASE Voltage from Vcurrent to Vmin

    if (stop->incSoftStart++ < SOFT_STOP_DURATION) {
        // If the current frequency is below MODULATION_IDEAL, decrement the voltage
        if (stop->addDeltaPhase > DELTA_IDEAL) {
            *voltage -= (uint8_t) stop->incVoltage;
        }
        // Always be decrementing the frequency
        if (stop->direction == FORWARD)
            *deltaPhase -= (uint16_t) stop->incDeltaPhase; // Now round
        else
            *deltaPhase -= UINT16_MAX - ((uint16_t) stop->incDeltaPhase); // Now round

        stop->addDeltaPhase += stop->incDeltaPhase;

        return false; // not finished
    } else
        return true; // finished
}

/**
 * ModVoltage controls pulse width
 *
 * Delta_Phase controls resultant sine wave frequency...how "far" to skip
 * through the sine lookup table for the next value.
 * Small steps = smaller frequency
 * Large steps = larger frequency
 *
 *
 * Current implementation takes 2.3ms for first pass and then subsequent ones tak 0.7ms
 * This function MUST be shorter time-wise than the SoftStart interval
 */
static bool DoSoftStart(STEP_MODULATION *start, uint8_t *voltage, uint16_t *deltaPhase) {
    // Enter here xx times to increase frequency by xxHz from xxHz up to xxHz
    // and increase Voltage from 0 to Vmax
    // Artifically assign an phase value
    // Starting phase point does not matter..only the delta
    if (++start->incSoftStart < SOFT_START_DURATION) {
        if (start->incSoftStart == 1) {
            // Get phase to a known state
            *deltaPhase = 0;
            *voltage = 0;
        } else if (start->incSoftStart == 2) {
            // Always start here at the min frequency
            *deltaPhase = DELTA_MIN;
            start->addDeltaPhase = DELTA_MIN;
            *voltage = AMPLITUDE_MIN;
            start->addVoltage = AMPLITUDE_MIN;
        } else {
            if (start->direction == FORWARD)
                *deltaPhase = (uint16_t) start->addDeltaPhase; // Now round
            else
                *deltaPhase = UINT16_MAX - ((uint16_t) start->addDeltaPhase); // Now round
            *voltage = (uint8_t) start->addVoltage; // Read max voltage level at nominal motor frequency of 60Hz

            volatile uint8_t temp = *voltage;
            temp = *deltaPhase;
            // Do floating point math at end so values are loaded first before TMR2 interrupt
            // Keep float point math accurate by adding the two together. Round at the end
            start->addDeltaPhase += start->incDeltaPhase;
            start->addVoltage += start->incVoltage;
        }
        return false; // not finished
    } else
        return true; // finished
}
